SystemOrganization addCategory: #'DBus-Objects'!

!DBusArgument methodsFor: '*dbus-objects' stamp: 'bf 5/1/2008 20:26'!
fromDBusArgument: sender
	"XXX refactor into subclasses"

	self isBasicType ifTrue: [
		type = DBusArgument objectPath
			ifTrue: [^sender dbusObjectForPath: value].
		^value].
	^type caseOf: {
		[DBusArgument struct]
			-> [value collect: [:ea | ea fromDBusArgument: sender]].
		[DBusArgument dictEntry]
			-> [(value key fromDBusArgument: sender) -> (value value fromDBusArgument: sender)].
		[DBusArgument array]
			-> [| arr |
				signature = 'ay' ifTrue: [^value].
				arr := value collect: [:ea | ea fromDBusArgument: sender].
				self containedSignature first =${
					ifTrue: [arr as: Dictionary]
					ifFalse: [arr]].
		[DBusArgument variant]
			-> [value fromDBusArgument: sender]
	}				   
			! !

DBusConnection subclass: #DBus
	instanceVariableNames: 'exported process replyHandlers matchHandlers accessLock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBus commentStamp: 'bf 6/11/2008 17:14' prior: 0!
I am a DBus connection. I retrieve messages in a background loop. Messages can be sent through me synchronously (that is, they block until a result comes in). I can export objects on the DBus to provide services for other applications.

FIXME: startup should only happen when I am used first, not at image startup time

(DBus sessionBus getObject: 'org.gnome.ScreenSaver' path: '/') SetActive: true!

!DBus methodsFor: 'mainloop' stamp: 'bf 6/19/2008 14:13'!
dispatchMessage: msg
	| object |
	object := self exportedAt: msg path.
	object ifNil: [
		^msg isMethodCall ifTrue: [
			self sendMessage: (DBusMessageError newFor: msg
				name: DBusMessageError dbusErrorUnknownObject
				withMessage: 'Unknown object ', msg path)]].

	object dbusHandle: msg from: self! !

!DBus methodsFor: 'exporting' stamp: 'bf 6/19/2008 14:12'!
exportedAt: aPathString
	^exported ifNotNil: [exported dbusChildAtPath: (aPathString findTokens: '/')].! !

!DBus methodsFor: 'exporting' stamp: 'bf 6/24/2008 18:06'!
export: aDBusObject on: aDBusName at: aPathString
	self registerName: aDBusName.
	exported ifNil: [exported := DBusObject new dbusConnection: self].
	exported dbusAdd: aDBusObject path: (aPathString findTokens: '/').! !

!DBus methodsFor: 'importing' stamp: 'bf 3/14/2008 19:54'!
getObject: aNameString path: aPathString
	^DBusProxy
		connection: self
		busName: aNameString
		objectPath: aPathString! !

!DBus methodsFor: 'importing' stamp: 'bf 4/30/2008 18:11'!
get: aProxyClass
	^aProxyClass new
		setConnection: self
		busName: aProxyClass dbusName
		objectPath: aProxyClass dbusPath! !

!DBus methodsFor: 'mainloop' stamp: 'bf 8/29/2008 16:07'!
handleMessage: msg
	| actions |
	self logDebug: self asString, ' received ', msg asString.
	"take care to not call handlers while accessLock is held, they could block"
	actions := OrderedCollection new.
	accessLock critical: [
		matchHandlers keysAndValuesDo: [:match :handler |
			(match matches: msg) ifTrue: [
				actions add: [handler copy valueWithArguments: {msg}] fixTemps]].
		msg isReplyOrError
			ifTrue: [(replyHandlers removeKey: msg replySerial ifAbsent: [])
				ifNotNilDo: [:handler | actions add: [handler handleReplyOrError: msg]]]
			ifFalse: [actions add: [self dispatchMessage: msg]]].
	actions do: [:action |	action	forkAt: Processor activePriority - 5
		named: self printString, ' handler ', msg serial asString]! !

!DBus methodsFor: 'logging' stamp: 'bf 5/1/2008 16:43'!
logDebug: aString
	WorldState addDeferredUIMessage: [Transcript cr; show: aString].! !

!DBus methodsFor: 'logging' stamp: 'bf 5/1/2008 16:39'!
logWarning: aString
	WorldState addDeferredUIMessage: [Transcript cr; show: aString].! !

!DBus methodsFor: 'mainloop' stamp: 'tfel 7/28/2016 12:01'!
mainloop
	[[true] whileTrue: 
		[self processMessages.
		readSemaphore wait]] on: Error do: [^ self]! !

!DBus methodsFor: 'matching' stamp: 'bf 8/29/2008 15:55'!
onMatch: aDBusMatch do: aBlockOrMessageSend
	accessLock critical: [
		self primAddMatch: aDBusMatch matchString squeakToUtf8.
		matchHandlers at: aDBusMatch put: aBlockOrMessageSend fixTemps]! !

!DBus methodsFor: 'mainloop' stamp: 'bf 6/19/2008 13:55'!
processMessages
	[self dataRemains] whileTrue:
			[self popMessage ifNotNilDo: [:msg | self handleMessage: msg]].
! !

!DBus methodsFor: 'matching' stamp: 'bf 6/19/2008 14:53'!
removeMatch: aDBusMatch
	accessLock critical: [
		self primRemoveMatch: aDBusMatch matchString squeakToUtf8.
		matchHandlers removeKey: aDBusMatch ifAbsent: []]! !

!DBus methodsFor: 'mainloop' stamp: 'bf 6/19/2008 13:53'!
restartMainloop
	process ifNotNil: [process terminate].
	process := [self mainloop]
		forkAt: Processor userInterruptPriority
		named: self printString, ' mainloop'! !

!DBus methodsFor: 'sending' stamp: 'bf 5/26/2008 17:42'!
sendDBusMessage: aMessage
	"send aMessage synchronously, wait until a response comes back. Answer reply message, or raise an error if send failed or timed out"

	^self sendDBusMessage: aMessage timeout: -1! !

!DBus methodsFor: 'sending' stamp: 'bf 6/19/2008 12:14'!
sendDBusMessage: aMessage timeout: seconds
	"send aMessage synchronously, wait until a response comes back. Answer reply message, or raise an error if send failed or timed out"

	| sema reply |
	sema := Semaphore new.
	self sendDBusMessage: aMessage
		timeout: seconds
		onSuccess: [:msg | reply := msg. sema signal]
		onError: [: msg | reply := msg. sema signal].
	sema wait.
	reply isError ifTrue: [^DBusError signal: reply message].
	^reply! !

!DBus methodsFor: 'sending' stamp: 'bf 6/19/2008 14:11'!
sendDBusMessage: aMessage timeout: seconds onSuccess: successHandler onError: errorHandler
	"send aMessage asynchronously, evaluating successHandler or errorHandler when a response or error comes in, or the timeout is over."
	
	| replyCode |
	accessLock critical: [
		replyCode := self sendMessage:  aMessage timeout: seconds.
		replyHandlers at: replyCode put: (DBusHandler onSuccess: successHandler onError: errorHandler)
	].
! !

!DBus methodsFor: 'private' stamp: 'bf 6/12/2008 16:25'!
sendMessage: msg timeout: seconds
	"Low-level send. Use sendDBusMessage:timeout: instead!!"
	self logDebug: self asString, ' sent ', msg asString.
	^super sendMessage: msg timeout: seconds! !

!DBus methodsFor: 'initialize-release' stamp: 'bf 6/19/2008 14:28'!
startUp
	exported := nil.
	replyHandlers := Dictionary new.
	matchHandlers := Dictionary new.
	accessLock := Semaphore forMutualExclusion.
	self restartMainloop.
	super startUp.
! !

Error subclass: #DBusError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

Object subclass: #DBusHandler
	instanceVariableNames: 'onSuccess onError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusHandler class methodsFor: 'instance creation' stamp: 'bf 6/10/2008 17:34'!
onSuccess: successHandler onError: errorHandler
	^self new
		onSuccess: successHandler;
		onError: errorHandler;
		yourself! !

!DBusHandler methodsFor: 'processing' stamp: 'bf 6/18/2008 16:20'!
handleReplyOrError: dbusReplyOrError

	(dbusReplyOrError isError ifTrue: [onError] ifFalse: [onSuccess])
		valueWithArguments: {dbusReplyOrError}

	"#valueWithArguments: works for both Blocks and MessageSends"! !

!DBusHandler methodsFor: 'accessing' stamp: 'bf 6/10/2008 16:22'!
onError
	"Answer the value of onError"

	^ onError! !

!DBusHandler methodsFor: 'accessing' stamp: 'bf 6/18/2008 16:19'!
onError: aBlockOrMessageSend
	onError := aBlockOrMessageSend! !

!DBusHandler methodsFor: 'accessing' stamp: 'bf 6/10/2008 16:22'!
onSuccess
	"Answer the value of onSuccess"

	^ onSuccess! !

!DBusHandler methodsFor: 'accessing' stamp: 'bf 6/18/2008 16:20'!
onSuccess: aBlockOrMessageSend
	onSuccess := aBlockOrMessageSend! !

Object subclass: #DBusMatch
	instanceVariableNames: 'matches argMatches'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusMatch methodsFor: 'private' stamp: 'bf 6/19/2008 14:25'!
argMatches
	^argMatches! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 19:46'!
arg: index is: anObject
	argMatches at: index put: anObject asString! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 18:39'!
destination: aString
	matches at: #destination put: aString! !

!DBusMatch methodsFor: 'setting-args' stamp: 'bf 6/18/2008 19:48'!
firstArg: anObject
	self arg: 0 is: anObject! !

!DBusMatch methodsFor: 'comparing' stamp: 'bf 6/19/2008 14:23'!
hash
	^matches hash bitXor: argMatches hash! !

!DBusMatch methodsFor: 'initialize' stamp: 'bf 6/18/2008 22:06'!
initialize
	matches := Dictionary new.
	argMatches := Dictionary new.
	self typeSignal! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 18:38'!
interface: aString
	matches at: #interface put: aString! !

!DBusMatch methodsFor: 'private' stamp: 'bf 6/19/2008 14:25'!
matches
	^matches! !

!DBusMatch methodsFor: 'matching' stamp: 'bf 6/18/2008 22:36'!
matches: aDBusMessage
	matches keysAndValuesDo: [:selector :value |
		(aDBusMessage perform: selector) = value
			ifFalse: [^false]].
	argMatches keysAndValuesDo: [:index :value |
		(aDBusMessage arguments at: index+1 ifAbsent: [^false])
			fromDBusArgument asString = value
				ifFalse: [^false]].
	^true! !

!DBusMatch methodsFor: 'printing' stamp: 'bf 6/18/2008 22:23'!
matchString
	^String streamContents: [:stream | self printMatchStringOn: stream]
! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 18:38'!
member: aString
	matches at: #member put: aString! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 18:39'!
path: aString
	matches at: #path put: aString! !

!DBusMatch methodsFor: 'printing' stamp: 'bf 6/18/2008 22:16'!
printMatchStringOn: aStream
	| first |
	first := true.
	matches keysAndValuesDo: [:selector :string |
		first ifTrue: [first := false] ifFalse: [aStream nextPut: $,].
		aStream
			nextPutAll: selector;
			nextPut: $=;
			print: string].
	argMatches keysAndValuesDo: [:index :string |
		first ifTrue: [first := false] ifFalse: [aStream nextPut: $,].
		aStream
			nextPutAll: 'arg';
			print: index;
			nextPut: $=;
			print: string].
! !

!DBusMatch methodsFor: 'printing' stamp: 'bf 6/18/2008 22:24'!
printOn: aStream
	aStream nextPutAll: self class name.
	aStream nextPut: $(.
	self printMatchStringOn: aStream.
	aStream nextPut: $)
! !

!DBusMatch methodsFor: 'setting-args' stamp: 'bf 6/18/2008 19:48'!
secondArg: anObject
	self arg: 1 is: anObject! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 18:37'!
sender: aString
	matches at: #sender put: aString! !

!DBusMatch methodsFor: 'setting-args' stamp: 'bf 6/18/2008 19:48'!
thirdArg: anObject
	self arg: 2 is: anObject! !

!DBusMatch methodsFor: 'setting-types' stamp: 'bf 6/19/2008 01:45'!
typeError
	self type: DBusMessageError type! !

!DBusMatch methodsFor: 'setting-types' stamp: 'bf 6/19/2008 01:46'!
typeMethodCall
	self type: DBusMessageMethodCall type! !

!DBusMatch methodsFor: 'setting-types' stamp: 'bf 6/19/2008 01:46'!
typeMethodReturn
	self type: DBusMessageMethodReply type! !

!DBusMatch methodsFor: 'setting-types' stamp: 'bf 6/19/2008 01:48'!
typeSignal
	self type: DBusMessageSignal type! !

!DBusMatch methodsFor: 'setting' stamp: 'bf 6/18/2008 19:53'!
type: aString
	"One of 'signal', 'method_call', 'method_return', 'error'. Defaults to signal"
	matches at: #type put: aString! !

!DBusMatch methodsFor: 'comparing' stamp: 'bf 6/19/2008 15:04'!
= other
	(other isKindOf: self class) ifFalse: [^false].
	^other matches = matches and: [other argMatches = argMatches]! !

Object subclass: #DBusMethod
	instanceVariableNames: 'interface member inSignature outSignature'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusMethod class methodsFor: 'instance creation' stamp: 'bf 5/16/2007 18:27'!
fromSpecString: aString
	^self new initializeFromSpecString: aString! !

!DBusMethod class methodsFor: 'instance creation' stamp: 'bf 6/19/2008 01:18'!
interface: anInterface member: aMember in: anInSignature out: anOutSignature
	^self new
		setInterface: anInterface
		member: aMember
		in: anInSignature
		out: anOutSignature! !

!DBusMethod methodsFor: 'accessing' stamp: 'bf 6/19/2008 01:16'!
fullSelector
	^interface, '.', member! !

!DBusMethod methodsFor: 'initialize-release' stamp: 'bf 6/19/2008 01:17'!
initializeFromSpecString: aString
	| p |
	interface := aString copyUpToLast: $..
	member := aString copyFrom: interface size+2 to: (p := aString indexOf: $<)-1.
	inSignature := aString copyFrom: p+1 to: (p := aString indexOf: $>)-1.
	outSignature := aString copyFrom: p+1 to: aString size.! !

!DBusMethod methodsFor: 'accessing' stamp: 'bf 2/25/2008 15:25'!
inSignature
	^inSignature! !

!DBusMethod methodsFor: 'accessing' stamp: 'bf 2/25/2008 15:24'!
interface
	^interface! !

!DBusMethod methodsFor: 'accessing' stamp: 'bf 6/19/2008 01:16'!
member
	^member! !

!DBusMethod methodsFor: 'accessing' stamp: 'bf 2/25/2008 15:25'!
outSignature
	^outSignature! !

!DBusMethod methodsFor: 'printing' stamp: 'bf 6/19/2008 01:18'!
printOn: aStream
	aStream
		nextPutAll: interface;
		nextPut: $.;
		nextPutAll: member;
		nextPut: $<;
		nextPutAll: inSignature;
		nextPut: $>;
		nextPutAll: outSignature! !

!DBusMethod methodsFor: 'printing' stamp: 'bf 6/19/2008 01:19'!
printXMLOn: xml
	xml startTag: 'method'; attribute: 'name' value: member; endTag.
	#('in' 'out') with: {inSignature. outSignature} do: [:direction :signature |
		DBusArgument signaturesIn: signature do: [:type |
			xml startTag: 'arg';
			attribute: 'direction' value: direction;
			attribute: 'type' value: type asString;
			endEmptyTag: 'arg']].
	xml endTag: 'method'.! !

!DBusMethod methodsFor: 'initialize-release' stamp: 'bf 6/19/2008 01:17'!
setInterface: anInterface member: aMember in: anInSignature out: anOutSignature
	interface := anInterface.
	member := aMember.
	inSignature := anInSignature.
	outSignature := anOutSignature.
! !

Object subclass: #DBusObject
	instanceVariableNames: 'dbusConnection dbusName dbusParent dbusChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusObject commentStamp: 'bf 5/1/2008 20:25' prior: 0!
I am an object exposed on the DBus.

Methods that are exported on the DBus are marked in the source using dbusMethod: and a string giving the DBus interface, selector, and signatures ('interface.method<inSignature>outSignature').

Most of my methods are prefixed with 'dbus' to minimize selector space polution for subclasses.
!

!DBusObject class methodsFor: 'instance creation' stamp: 'bf 5/21/2007 17:46'!
parent: aDBusObject name: aString
	^self new dbusParent: aDBusObject name: aString! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 2/21/2008 13:47'!
dbusAdd: aDBusObject path: aPathArray

	aPathArray size > 1 ifTrue: [
		^(dbusChildren at: aPathArray first ifAbsentPut: [DBusObject parent: self name: aPathArray first])
			dbusAdd: aDBusObject path: aPathArray allButFirst].

	aDBusObject dbusParent: self name: aPathArray last.

	dbusChildren at: aDBusObject dbusName put: aDBusObject! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 6/11/2008 17:30'!
dbusChildAtPath: aPathArray
	^aPathArray size = 0
		ifTrue: [self]
		ifFalse: [(self dbusChildren at: aPathArray first ifAbsent: [^nil])
			dbusChildAtPath: aPathArray allButFirst]! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 6/11/2008 17:29'!
dbusChildren
	^dbusChildren! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 6/18/2008 16:37'!
dbusCoerceTo: type
	^self dbusPath! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/21/2007 17:45'!
dbusConnection
	^dbusConnection! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/21/2007 17:48'!
dbusConnection: aDBusConnection
	dbusConnection := aDBusConnection! !

!DBusObject methodsFor: 'handling' stamp: 'bf 6/19/2008 01:15'!
dbusHandle: aMessage from: aDBus
	"aMessage for me was received on the D-Bus. Go through my dbusMethods and dispatch to the right one. Then collect the return values, pack them as DBusArguments and send a reply.".

	self dbusMethodSpecsAndSelectorsDo: [:spec :sel |
		(aMessage interface
			ifNil: [((spec copyUpTo: $<) copyAfterLast: $.) = aMessage member]
			ifNotNil: [(spec copyUpTo: $<) = aMessage fullSelector])
		ifTrue: [^self dbusHandle: aMessage from: aDBus spec: spec selector: sel]].

	"no method found"	
	aMessage isMethodCall ifTrue: [
		aDBus sendMessage: (DBusMessageError newFor: aMessage
			name: DBusMessageError dbusErrorUnknownMethod
			withMessage: 'Unknown method ', aMessage fullSelector, '()')].
! !

!DBusObject methodsFor: 'handling' stamp: 'bf 6/17/2008 13:57'!
dbusHandle: aMessage from: aDBus spec: aSpecString selector: aSelector
	^self dbusHandle: aMessage from: aDBus spec: aSpecString selector: aSelector in: self! !

!DBusObject methodsFor: 'handling' stamp: 'bf 6/17/2008 14:06'!
dbusHandle: aMessage from: aDBus spec: aSpecString selector: aSelector in: anObject
	| result |
	[
		result := anObject perform: aSelector withArguments: (aMessage arguments collect: [:arg | arg fromDBusArgument: self]).
		aMessage reply ifTrue: [self dbusSendReply: result for: aMessage from: aDBus spec: aSpecString]
	] on: Error do: [:error | 
		aDBus sendMessage: (DBusMessageError newFor: aMessage
			name: 'org.squeak.error.', error class name
			withMessage: error messageText).
		error pass
	]! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 2/29/2008 11:39'!
dbusInterfaces
	"self basicNew dbusInterfaces"
	| interfaces |
	interfaces := Set new.
	self dbusMethodSpecsAndSelectorsDo: [:spec :selector |
		interfaces add: (spec copyUpToLast: $.)].
	^interfaces
! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/16/2007 18:29'!
dbusInterface: interface methodsDo: aBlock
	self dbusMethodSpecsAndSelectorsDo: [:spec :selector |
		interface = (spec copyUpToLast: $.)
			ifTrue: [aBlock value: (DBusMethod fromSpecString: spec)]]
! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 8/17/2010 13:16'!
dbusMethodSpecsAndSelectorsDo: aBlock
	"Evaluate aBlock with all method specs and corresponding selectors that are to be exposed on the DBus.
	By default, this gathers only methods marked as #dbusMethod:."
	| cls |
	cls := self class.
	[cls selectorsAndMethodsDo: [:sel :meth | 
		(meth numLiterals >= 2 and: [(meth literalAt: 1) == #dbusMethod: and: [(meth literalAt: 2) isString]])
			ifTrue: [aBlock value: (meth literalAt: 2) value: sel]].
	cls == DBusObject] whileFalse: [cls := cls superclass].
! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 6/19/2008 01:16'!
dbusMethods
	"Answer a Dictionary mapping DBus selectors to DBusMethod instances"
	| methods method |
	methods := Dictionary new.
	self dbusMethodSpecsAndSelectorsDo: [:spec :selector |
		method := DBusMethod fromSpecString: spec.
		methods at: method member put: method].
	^methods
! !

!DBusObject methodsFor: 'exporting' stamp: 'bf 2/29/2008 11:45'!
dbusMethod: formatString
	"This message marks a method to be exported on the DBus. It must be the first message-send in the method.
	The format is
		'interface.method<inSignature>outSignature'
	where both inSignature and outSignature may be empty"
	
	^self
! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/16/2007 17:45'!
dbusName
	^dbusName! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 6/19/2008 14:13'!
dbusObjectForPath: aString
	^(dbusConnection exportedAt: aString) ifNil: [aString]! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/21/2007 18:05'!
dbusParent
	^dbusParent! !

!DBusObject methodsFor: 'initialize-release' stamp: 'bf 5/21/2007 17:53'!
dbusParent: aDBusObject name: aString
	dbusConnection := aDBusObject dbusConnection.
	dbusParent := aDBusObject.
	dbusName := aString.
! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/21/2007 18:07'!
dbusPath
	^dbusParent
		ifNil: ['/']
		ifNotNil: [
			dbusParent dbusParent
				ifNil: ['/', dbusName]
				ifNotNil: [dbusParent dbusPath, '/', dbusName]]! !

!DBusObject methodsFor: 'handling' stamp: 'bf 6/17/2008 16:02'!
dbusSendReply: anObject for: aMessage from: aDBus spec: aSpecString
	| reply signatures |
	reply := DBusMessage newReplyFor: aMessage.
	signatures := DBusArgument splitSignature: (aSpecString copyAfterLast: $>).
	signatures size = 1 ifTrue: [reply addArgument: (anObject asDBusArgumentSignature: signatures first)].
	signatures size > 1 ifTrue: [reply with: signatures do: [:val :sig | reply addArgument: (val asDBusArgumentSignature: sig)]].
	aDBus sendMessage: reply! !

!DBusObject methodsFor: 'accessing' stamp: 'bf 5/25/2007 21:08'!
dbusType
	^DBusArgument objectPath
! !

!DBusObject methodsFor: 'initialize-release' stamp: 'bf 5/21/2007 17:53'!
initialize
	dbusName := ''.
	dbusParent := nil.
	dbusChildren := Dictionary new.
! !

!DBusObject methodsFor: 'dbus methods' stamp: 'bf 6/16/2008 19:18'!
introspect
	self dbusMethod: 'org.freedesktop.DBus.Introspectable.Introspect<>s'.

	^String streamContents: [:stream |
		| xml |
		stream nextPutAll: '<!!DOCTYPE node PUBLIC "-//freedesktop//DTD D-BUS Object Introspection 1.0//EN" "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">'.
		xml := XMLWriter on: stream.
		xml startTag: 'node'; endTag.
		self dbusChildren do: [:child |
			xml startTag: 'node'; attribute: 'name' value: child dbusName; endEmptyTag: 'node'].
		self dbusInterfaces do: [:interface |
			interface = 'org.freedesktop.DBus.Introspectable' ifFalse: [
				xml startTag: 'interface'; attribute: 'name' value: interface; endTag.
				self dbusInterface: interface methodsDo: [:method |
					method printXMLOn: xml].
				xml endTag: 'interface'	]].
		xml endTag: 'node']! !

!DBusObject methodsFor: 'printing' stamp: 'bf 5/21/2007 17:56'!
printOn: aStream
	aStream
		print: self class;
		nextPut: $(;
		nextPutAll: self dbusPath;
		nextPut: $)! !

Object subclass: #DBusProxy
	instanceVariableNames: 'connection busName objectPath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusProxy commentStamp: 'bf 3/27/2008 16:31' prior: 0!
I am a proxy for an object on the DBus. You can send messages using #dbusPerform: and its variants.
!

DBusProxy subclass: #DBusCompiledProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

DBusCompiledProxy class
	instanceVariableNames: 'current'!

!DBusCompiledProxy commentStamp: '<historical>' prior: 0!
I'm the superclass for a DBus proxy with delegation methods compiled by introspection.
!

DBusCompiledProxy class
	instanceVariableNames: 'current'!

!DBusCompiledProxy class methodsFor: 'accessing' stamp: 'bf 3/14/2008 19:40'!
classForName: aDBusName andPath: aDBusPath
	self allSubclassesDo: [:cls |
		(cls handlesDBusName: aDBusName andPath: aDBusPath)
			ifTrue: [^cls]].
	^nil! !

!DBusCompiledProxy class methodsFor: 'compiling' stamp: 'bf 7/11/2008 20:10'!
compileMethodsAndSignalsFrom: introspectionString
	| node |
	node := (XMLDOMParser parseDocumentFrom: introspectionString readStream) elements first.
	node tagsNamed: #interface do: [:interface |
		(interface attributeAt: 'name') = 'org.freedesktop.DBus.Introspectable' ifFalse: [
			interface tagsNamed: #method do: [:method |
				self compileMethod: (method attributeAt: 'name')
					interface: (interface attributeAt: 'name')
					in: (self parameters: 'in' from: method)
					out: (self parameters: 'out' from: method)].
			interface tagsNamed: #signal do: [:signal |
				self compileSignal: (signal attributeAt: 'name')
					interface: (interface attributeAt: 'name')
					in: (self parameters: 'in' from: signal)]]].

! !

!DBusCompiledProxy class methodsFor: 'compiling' stamp: 'bf 3/27/2008 15:50'!
compileMethod: memberString interface: interfaceString in: inParams out: outParams
	| overwriteOkay newSource oldSource |
	overwriteOkay := '"Automatically generated"'.
	newSource := String streamContents: [:strm |
		strm nextPutAll: (self underscoreToCamelCase: memberString).
		inParams withIndexDo: [:param :i |
			i=1 ifTrue: [strm nextPutAll: ': ']
				ifFalse: [strm nextPutAll: ' with: '].
			strm nextPutAll: param key].

		strm crtab; nextPutAll: '"Returns '.
		outParams 
			ifEmpty: [strm nextPutAll: 'nothing']
			ifNotEmpty: [
				outParams do: [:p | strm nextPutAll: p key, ' ''', p value, '''']
				separatedBy: [strm nextPutAll: ' and ']].
		strm nextPutAll: '."'.

		strm crtab; nextPutAll: overwriteOkay.

		strm crtab; nextPutAll: '^self dbusPerform: '; print: memberString; 
			nextPutAll: ' interface: '; print: interfaceString.
		inParams isEmpty ifFalse: [
			strm nextPutAll: ' withArguments: {'.
			inParams do: [:param |
				strm crtab: 2; 
					nextPutAll: 'DBusArgument value: '; nextPutAll: param key;
					nextPutAll: ' signature: '; print: param value;
					nextPutAll: '.'].
			strm nextPutAll: '}']].

	oldSource := (self sourceCodeAt: (Parser new parseSelector: newSource) asSymbol
		ifAbsent: [overwriteOkay]) asString.
	(oldSource ~= newSource and: [oldSource includesSubString: overwriteOkay])
		ifTrue: [self compile: newSource classified: interfaceString].! !

!DBusCompiledProxy class methodsFor: 'compiling' stamp: 'bf 7/11/2008 20:36'!
compileSignal: memberString interface: interfaceString in: inParams
	| overwriteOkay newSource oldSource |
	overwriteOkay := '"Automatically generated"'.
	newSource := String streamContents: [:strm |
		strm nextPutAll: 'on', (self underscoreToCamelCase: memberString) capitalized,
			'Send: aSelector to: anObject'.
		strm crtab; nextPutAll: overwriteOkay.
		strm crtab; nextPutAll: '^self onDBusSignal: '; print: memberString; 
			crtab: 2; nextPutAll: 'interface: '; print: interfaceString;
			crtab: 2; nextPutAll: 'signature: '''.
		inParams do: [:param | strm nextPutAll: param value].
		strm nextPutAll: '''';
			crtab: 2; nextPutAll: 'send: aSelector to: anObject'].

	oldSource := (self sourceCodeAt: (Parser new parseSelector: newSource) asSymbol
		ifAbsent: [overwriteOkay]) asString.
	(oldSource ~= newSource and: [oldSource includesSubString: overwriteOkay])
		ifTrue: [self compile: newSource classified: interfaceString].! !

!DBusCompiledProxy class methodsFor: 'accessing' stamp: 'bf 3/14/2008 19:27'!
dbusConnection
	^ self subclassResponsibility ! !

!DBusCompiledProxy class methodsFor: 'accessing' stamp: 'bf 3/3/2008 17:55'!
dbusName
	^ self subclassResponsibility ! !

!DBusCompiledProxy class methodsFor: 'accessing' stamp: 'bf 3/3/2008 17:56'!
dbusPath
	^ self subclassResponsibility ! !

!DBusCompiledProxy class methodsFor: 'testing' stamp: 'bf 3/27/2008 12:41'!
handlesDBusName: aDBusName andPath: aDBusPath
	^self dbusName = aDBusName and: [self dbusPath match: aDBusPath]! !

!DBusCompiledProxy class methodsFor: 'testing' stamp: 'bf 7/7/2008 15:29'!
isAbstract
	^self == DBusCompiledProxy! !

!DBusCompiledProxy class methodsFor: 'compiling' stamp: 'bf 3/27/2008 16:16'!
parameterFromSignature: aSignature
	| parameter |
	parameter := DBusArgument nameOfSignature: aSignature.
	^parameter first isVowel
		ifTrue: ['an', parameter]
		ifFalse: ['a', parameter].! !

!DBusCompiledProxy class methodsFor: 'compiling' stamp: 'bf 3/27/2008 16:16'!
parameters: direction from: aNode
	"given introspection data for a method or signal node, format a parameter list like #(nameA->type nameB->type)"

	| parameters signatures  parameter signature |
	parameters := OrderedCollection new.
	signatures := OrderedCollection new.

	aNode tagsNamed: #arg do: [:arg |
		(arg attributeAt: 'direction' ifAbsent: ['in']) = direction ifTrue: [
			signature := arg attributeAt: 'type'.
			parameter := arg attributeAt: 'name' ifAbsent: [nil].
			parameter := parameter
				ifNil: [self parameterFromSignature: signature]
				ifNotNil: [(self underscoreToCamelCase: parameter), 
					(DBusArgument nameOfSignature: signature)].
			[self allInstVarNames includes: parameter]
				whileTrue: [
					parameter := parameter first isVowel
						ifTrue: ['an', parameter capitalized]
						ifFalse: ['a', parameter capitalized]].
			signatures add: signature.
			parameters add: parameter]].

	"Append letter if necessary for uniqueness"
	parameters := parameters withIndexCollect: [:param :i |
		(parameters occurrencesOf: param) = 1
			ifTrue: [param]
			ifFalse: [param copyWith: (Character value: $A asInteger+
				((parameters first: i) occurrencesOf: param)-1)]].

	^parameters with: signatures collect: [:p :s | p -> s]! !

!DBusCompiledProxy class methodsFor: 'compiling' stamp: 'bf 3/28/2008 13:16'!
underscoreToCamelCase: aString
	| s |
	s := aString.
	(s allSatisfy: [:c | c = $_ or: [c isUppercase]])
		ifTrue: [s := s asLowercase].
 	^(s copyReplaceAll: '_' with: ' ')
		toCamelCase withFirstCharacterDownshifted! !

!DBusCompiledProxy methodsFor: 'accessing' stamp: 'bf 3/14/2008 19:27'!
dbusConnection
	^super dbusConnection ifNil: [self class dbusConnection]! !

!DBusCompiledProxy methodsFor: 'accessing' stamp: 'bf 3/14/2008 19:25'!
dbusName
	^super dbusName ifNil: [self class dbusName]! !

!DBusCompiledProxy methodsFor: 'accessing' stamp: 'bf 3/14/2008 19:26'!
dbusPath
	^super dbusPath ifNil: [self class dbusPath]! !

DBusCompiledProxy subclass: #DBusDaemon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusDaemon commentStamp: '<historical>' prior: 0!
I am an example compiled proxy for the 'org.freedesktop.DBus' service. My methods were generated like this:

	DBusDaemon compileMethodsFrom: DBusDaemon current introspect

I can, e.g., list names on the DBus:

	DBusDaemon current listNames!

!DBusDaemon class methodsFor: 'accessing' stamp: 'bf 3/27/2008 15:37'!
dbusName
	^'org.freedesktop.DBus'! !

!DBusDaemon class methodsFor: 'accessing' stamp: 'bf 3/27/2008 15:37'!
dbusPath
	^'/'! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
addMatch: aString
	"Returns nothing."
	"Automatically generated"
	^self dbusPerform: 'AddMatch' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 16:06'!
getConnectionSELinuxSecurityContext: aString
	"Returns anArrayOfBytes 'ay'."
	"Automatically generated"
	^self dbusPerform: 'GetConnectionSELinuxSecurityContext' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
getConnectionUnixProcessID: aString
	"Returns anUint32 'u'."
	"Automatically generated"
	^self dbusPerform: 'GetConnectionUnixProcessID' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
getConnectionUnixUser: aString
	"Returns anUint32 'u'."
	"Automatically generated"
	^self dbusPerform: 'GetConnectionUnixUser' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
getNameOwner: aString
	"Returns aString 's'."
	"Automatically generated"
	^self dbusPerform: 'GetNameOwner' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
hello
	"Returns aString 's'."
	"Automatically generated"
	^self dbusPerform: 'Hello' interface: 'org.freedesktop.DBus'! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 16:06'!
listActivatableNames
	"Returns anArrayOfStrings 'as'."
	"Automatically generated"
	^self dbusPerform: 'ListActivatableNames' interface: 'org.freedesktop.DBus'! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 16:06'!
listNames
	"Returns anArrayOfStrings 'as'."
	"Automatically generated"
	^self dbusPerform: 'ListNames' interface: 'org.freedesktop.DBus'! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 16:06'!
listQueuedOwners: aString
	"Returns anArrayOfStrings 'as'."
	"Automatically generated"
	^self dbusPerform: 'ListQueuedOwners' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
nameHasOwner: aString
	"Returns aBoolean 'b'."
	"Automatically generated"
	^self dbusPerform: 'NameHasOwner' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
releaseName: aString
	"Returns anUint32 'u'."
	"Automatically generated"
	^self dbusPerform: 'ReleaseName' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
reloadConfig
	"Returns nothing."
	"Automatically generated"
	^self dbusPerform: 'ReloadConfig' interface: 'org.freedesktop.DBus'! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
removeMatch: aString
	"Returns nothing."
	"Automatically generated"
	^self dbusPerform: 'RemoveMatch' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
requestName: aString with: anUint32
	"Returns anUint32 'u'."
	"Automatically generated"
	^self dbusPerform: 'RequestName' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.
		DBusArgument value: anUint32 signature: 'u'.}! !

!DBusDaemon methodsFor: 'org.freedesktop.DBus' stamp: 'bf 3/27/2008 15:52'!
startServiceByName: aString with: anUint32
	"Returns anUint32 'u'."
	"Automatically generated"
	^self dbusPerform: 'StartServiceByName' interface: 'org.freedesktop.DBus' withArguments: {
		DBusArgument value: aString signature: 's'.
		DBusArgument value: anUint32 signature: 'u'.}! !

DBusProxy subclass: #DBusGenericProxy
	instanceVariableNames: 'methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DBus-Objects'!

!DBusGenericProxy commentStamp: '<historical>' prior: 0!
I am a generic proxy for an object on the DBus. You can simply send Smalltalk messages to me that I intercept via #doesNotUnderstand: and forward to the DBus.!

!DBusGenericProxy methodsFor: 'accessing' stamp: 'bf 2/29/2008 11:35'!
dbusMethods
	^methods ifNil: [
		methods := [super dbusMethods] ifError: [Dictionary new]]! !

!DBusGenericProxy methodsFor: 'calling' stamp: 'bf 2/29/2008 11:20'!
doesNotUnderstand: aMessage
	aMessage selector first isLetter ifFalse: [^super doesNotUnderstand: aMessage].
	^self dbusPerform: (aMessage selector copyUpTo: $:) withArguments: aMessage arguments
! !

!DBusGenericProxy methodsFor: 'testing' stamp: 'bf 2/29/2008 11:29'!
useIntrospection
	^true! !

!DBusProxy class methodsFor: 'instance creation' stamp: 'bf 7/7/2008 15:28'!
connection: aDBusConnection busName: aNameString objectPath: aPathString
	"Create a CompiledProxy for the given name and path, or a GenericProxy"
	| proxyClass |
	proxyClass := self isAbstract
		ifFalse: [self]
		ifTrue: [(DBusCompiledProxy classForName: aNameString andPath: aPathString)
			ifNil: [DBusGenericProxy]].
	^proxyClass new
		setConnection: aDBusConnection
		busName: aNameString
		objectPath: aPathString.
! !

!DBusProxy class methodsFor: 'testing' stamp: 'bf 7/7/2008 15:28'!
isAbstract
	^self == DBusProxy! !

!DBusProxy methodsFor: 'signals' stamp: 'bf 7/7/2008 21:58'!
assertDBusSignature: aString matchesSelector: aSymbol
	self assert: [
		| argCount |
		argCount := aSymbol occurrencesOf: $:.
		DBusArgument signaturesIn: aString
			do: [:sig | argCount := argCount - 1].
		argCount = 0]! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 6/18/2008 16:38'!
dbusCoerceTo: type
	^ self dbusPath! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 3/27/2008 17:27'!
dbusConnection
	^connection! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 6/19/2008 01:18'!
dbusMethods
	"Answer a Dictionary mapping DBus selectors to DBusMethod instances"
	| introspected types |
	introspected := Dictionary new.
	(XMLDOMParser parseDocumentFrom: self introspect readStream) elements first
		tagsNamed: #interface do: [:interface |
			interface tagsNamed: #method do: [:method |
				types := {'in' -> String new writeStream.
					'out' -> String new writeStream} as: Dictionary.
				method tagsNamed: #arg do: [:arg |
					(types at: (arg attributeAt: 'direction'))
						nextPutAll:  (arg attributeAt: 'type')].
				introspected
					at: (method attributeAt: 'name')
					put: (DBusMethod
						interface: (interface attributeAt: 'name')
						member: (method attributeAt: 'name')
						in: (types at: 'in') contents
						out: (types at: 'out') contents)]].
	^introspected
! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 2/29/2008 11:53'!
dbusName
	^busName! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 5/1/2008 20:27'!
dbusObjectForPath: aPath
	^DBusProxy
		connection: self dbusConnection
		busName: self dbusName
		objectPath: aPath! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 2/29/2008 11:54'!
dbusPath
	^objectPath! !

!DBusProxy methodsFor: 'calling' stamp: 'bf 6/19/2008 21:12'!
dbusPerformAsync: memberString interface: interfaceString
	^self dbusPerformAsync: memberString interface: interfaceString withArguments: #()
! !

!DBusProxy methodsFor: 'calling' stamp: 'bf 6/19/2008 21:14'!
dbusPerformAsync: memberString interface: interfaceString withArguments: argumentArray
	| msg |
	msg  := DBusMessageMethodCall
		destination: self dbusName
		path: self dbusPath
		interface: interfaceString
		member: memberString
		reply: false.
	argumentArray do: [:each | msg addArgument: each].
	self dbusConnection sendMessage: msg.
	^nil! !

!DBusProxy methodsFor: 'calling' stamp: 'bf 2/26/2008 11:31'!
dbusPerform: interfaceAndSelector
	"interfaceAndSelector is either 'interface.selector' or simply 'selector'. In the latter case, the interface will be inferred by introspection."
	^self dbusPerform: interfaceAndSelector withArguments: #()! !

!DBusProxy methodsFor: 'calling' stamp: 'bf 2/25/2008 14:53'!
dbusPerform: selectorString interface: interfaceString
	^self dbusPerform: selectorString interface: interfaceString withArguments: #()
! !

!DBusProxy methodsFor: 'calling' stamp: 'bf 6/19/2008 01:12'!
dbusPerform: memberString interface: interfaceString withArguments: argumentArray
	| msg reply |
	msg  := DBusMessageMethodCall
		destination: self dbusName
		path: self dbusPath
		interface: interfaceString
		member: memberString.
	argumentArray do: [:each | msg addArgument: each].
	reply := self dbusConnection sendDBusMessage: msg.
	reply arguments size = 0 ifTrue: [^nil].
	reply arguments size = 1 ifTrue: [^reply arguments first fromDBusArgument: self].
	^reply arguments collect: [:each | each fromDBusArgument: self]! !

!DBusProxy methodsFor: 'calling' stamp: 'bf 2/29/2008 11:48'!
dbusPerform: interfaceAndSelector withArguments: argumentArray
	"interfaceAndSelector is either 'interface.selector' or simply 'selector'."
	| selector interface dot |
	dot := interfaceAndSelector lastIndexOf: $..
	dot > 0
		ifTrue: [selector := interfaceAndSelector allButFirst: dot.
			interface := interfaceAndSelector first: dot-1.
			interface isEmpty ifTrue: [interface := nil]]
		ifFalse: [
			selector := interfaceAndSelector.
			interface := nil.
			self useIntrospection ifTrue: [
				self dbusMethods at: selector
					ifPresent: [:method | interface := method interface]]].
	^self dbusPerform: selector interface: interface withArguments: argumentArray
! !

!DBusProxy methodsFor: 'accessing' stamp: 'bf 5/25/2007 20:14'!
dbusType
	^DBusArgument objectPath
! !

!DBusProxy methodsFor: 'org.freedesktop.DBus.Introspectable' stamp: 'bf 3/27/2008 18:01'!
introspect
	"Returns aString 's'."
	"Automatically generated - not regenerated in subclasses"
	^self dbusPerform: 'Introspect' interface: 'org.freedesktop.DBus.Introspectable'! !

!DBusProxy methodsFor: 'signals' stamp: 'bf 9/9/2008 14:37'!
onDBusSignal: memberString interface: interfaceString arguments: argMatches send: aSelector to: anObject
	"argMatches are zero-based: {0->'some string'} matches the first argument"
	| match |
	match := DBusMatch new.
	match path: self dbusPath.
	memberString ifNotNil: [match member: memberString].
	interfaceString ifNotNil: [match interface: interfaceString].
	argMatches do: [:arg | match arg: arg key is: arg value].
	aSelector ifNil: [self dbusConnection removeMatch: match. ^match].
	self dbusConnection
		onMatch: match
		do: [:msg | anObject perform: aSelector
			withEnoughArguments:
				(msg arguments collect: [:each | each fromDBusArgument: self])].
	^match! !

!DBusProxy methodsFor: 'signals' stamp: 'bf 9/9/2008 14:19'!
onDBusSignal: memberString interface: interfaceString send: aSelector to: anObject 
	^self onDBusSignal: memberString interface: interfaceString arguments: #() send: aSelector to: anObject ! !

!DBusProxy methodsFor: 'signals' stamp: 'bf 7/11/2008 19:53'!
onDBusSignal: memberString interface: interfaceString signature: signatureString send: aSelector to: anObject 

	signatureString ifNotNil: [
		self assertDBusSignature: signatureString matchesSelector: aSelector].
	^self onDBusSignal: memberString interface: interfaceString send: aSelector to: anObject 
! !

!DBusProxy methodsFor: 'signals' stamp: 'bf 7/7/2008 21:38'!
onDBusSignal: interfaceAndMember send: aSelector to: anObject 
	| member interface dot |
	dot := interfaceAndMember lastIndexOf: $..
	dot > 0
		ifTrue: [member := interfaceAndMember allButFirst: dot.
			interface := interfaceAndMember first: dot-1]
		ifFalse: [member := interfaceAndMember.
			interface := nil].
	^self onDBusSignal: member
		interface: interface
		send: aSelector to: anObject ! !

!DBusProxy methodsFor: 'printing' stamp: 'bf 6/19/2008 12:23'!
printOn: aStream
	aStream
		print: self class;
		nextPut: $(;
		print: self  dbusName;
		space;
		print: self dbusPath;
		nextPut: $)! !

!DBusProxy methodsFor: 'initialize-release' stamp: 'bf 2/25/2008 13:42'!
setConnection: aDBusConnection busName: aNameString objectPath: aPathString
	connection := aDBusConnection.
	busName := aNameString.
	objectPath := aPathString.
! !

!DBusProxy methodsFor: 'testing' stamp: 'bf 2/29/2008 11:29'!
useIntrospection
	^false! !
